@@ -70,7 +70,7 @@ struct pcg_prng {
   pcg_prng(uint64_t seed) { gen.seed(seed); }
 
   // return a value in (0, n]
-  inline std::size_t operator()(std::size_t n) {
+  inline std::size_t operator()(std::size_t n, std::size_t, std::size_t) {
     std::size_t result = gen(n);
     return result;
   }
@@ -129,4 +129,14 @@ struct pcg_factory {
   }
 };
 
+struct deterministic_factory {
+  deterministic_factory(std::size_t) {}
+
+  void reseed() {}
+
+  uwot::deterministic_ng create(std::size_t seed) {
+    return uwot::deterministic_ng();
+  }
+};
+
 #endif // UWOT_RNG_H
